@@ -5,11 +5,13 @@ import json
 import os
 from tempfile import TemporaryDirectory
 
+import pytest
 from jupyter_core.application import JupyterApp
 from jupyter_core.paths import jupyter_runtime_dir
 
 from jupyter_client import connect
 from jupyter_client import KernelClient
+from jupyter_client import KernelManager
 from jupyter_client.consoleapp import JupyterConsoleApp
 from jupyter_client.session import Session
 
@@ -35,8 +37,8 @@ class TemporaryWorkingDirectory(TemporaryDirectory):
 
 
 class DummyConsoleApp(JupyterApp, JupyterConsoleApp):
-    def initialize(self, argv=[]):
-        JupyterApp.initialize(self, argv=argv)
+    def initialize(self, argv=None):
+        JupyterApp.initialize(self, argv=argv or [])
         self.init_connection_file()
 
 
@@ -84,7 +86,7 @@ def test_write_connection_file():
 
 
 def test_load_connection_file_session():
-    """test load_connection_file() after """
+    """test load_connection_file() after"""
     session = Session()
     app = DummyConsoleApp(session=Session())
     app.initialize(argv=[])
@@ -101,7 +103,7 @@ def test_load_connection_file_session():
 
 
 def test_load_connection_file_session_with_kn():
-    """test load_connection_file() after """
+    """test load_connection_file() after"""
     session = Session()
     app = DummyConsoleApp(session=Session())
     app.initialize(argv=[])
@@ -235,3 +237,59 @@ def test_mixin_cleanup_random_ports():
         assert not os.path.exists(filename)
         for name in dc._random_port_names:
             assert getattr(dc, name) == 0
+
+
+param_values = [
+    (True, True),
+    (True, False),
+    (False, True),
+    (False, False),
+]
+
+
+@pytest.mark.parametrize("file_exists, km_matches", param_values)
+def test_reconcile_connection_info(file_exists, km_matches):
+
+    expected_info = sample_info
+    mismatched_info = sample_info.copy()
+    mismatched_info["key"] = b"def456"
+    mismatched_info["shell_port"] = expected_info["shell_port"] + 42
+    mismatched_info["control_port"] = expected_info["control_port"] + 42
+
+    with TemporaryDirectory() as connection_dir:
+
+        cf = os.path.join(connection_dir, "kernel.json")
+        km = KernelManager()
+        km.connection_file = cf
+
+        if file_exists:
+            _, info = connect.write_connection_file(cf, **expected_info)
+            info["key"] = info["key"].encode()  # set 'key' back to bytes
+
+            if km_matches:
+                # Let this be the case where the connection file exists, and the KM has matching
+                # values prior to reconciliation.  This is the LocalProvisioner case.
+                provisioner_info = info
+                km.load_connection_info(provisioner_info)
+            else:
+                # Let this be the case where the connection file exists, and the KM has those values
+                # that differ from the ones returned by the provisioner.  This is the restart-with-
+                # changed-ports case (typical for remote provisioners).
+                km.load_connection_info(expected_info)
+                provisioner_info = mismatched_info
+        else:  # connection file does not exist
+            if km_matches:
+                # Let this be the case where the connection file does not exist, NOR does the KM
+                # have any values of its own and reconciliation sets those values. This is the
+                # non-LocalProvisioner case.
+                provisioner_info = expected_info
+            else:
+                # Let this be the case where the connection file does not exist, yet the KM
+                # has values that do not match those returned from the provisioner.  This case
+                # is probably not practical and is equivalent to the True, False case.
+                km.load_connection_info(expected_info)
+                provisioner_info = mismatched_info
+
+        km._reconcile_connection_info(provisioner_info)
+        km_info = km.get_connection_info()
+        assert km._equal_connections(km_info, provisioner_info)
