@@ -45,7 +45,7 @@ def config(transport):
 
 def _install_kernel(name="signaltest", extra_env=None):
     if extra_env is None:
-        extra_env = dict()
+        extra_env = {}
     kernel_dir = pjoin(paths.jupyter_data_dir(), "kernels", name)
     os.makedirs(kernel_dir)
     with open(pjoin(kernel_dir, "kernel.json"), "w") as f:
@@ -164,15 +164,17 @@ class TestKernelManagerShutDownGracefully:
         kc.stop_channels()
         km.shutdown_kernel()
 
-        assert km._shutdown_status == expected
+        if expected == _ShutdownStatus.ShutdownRequest:
+            expected = [expected, _ShutdownStatus.SigtermRequest]
+        else:
+            expected = [expected]
+
+        assert km._shutdown_status in expected
 
     @pytest.mark.asyncio
     @pytest.mark.skipif(sys.platform == "win32", reason="Windows doesn't support signals")
     @pytest.mark.parametrize(*parameters)
     async def test_async_signal_kernel_subprocesses(self, name, install, expected):
-        # ipykernel doesn't support 3.6 and this test uses async shutdown_request
-        if expected == _ShutdownStatus.ShutdownRequest and sys.version_info < (3, 7):
-            pytest.skip()
         install()
         km, kc = await start_new_async_kernel(kernel_name=name)
         assert km._shutdown_status == _ShutdownStatus.Unset
@@ -181,12 +183,18 @@ class TestKernelManagerShutDownGracefully:
         kc.stop_channels()
         await km.shutdown_kernel()
 
-        assert km._shutdown_status == expected
+        if expected == _ShutdownStatus.ShutdownRequest:
+            expected = [expected, _ShutdownStatus.SigtermRequest]
+        else:
+            expected = [expected]
+
+        assert km._shutdown_status in expected
 
 
 class TestKernelManager:
     def test_lifecycle(self, km):
         km.start_kernel(stdout=PIPE, stderr=PIPE)
+        kc = km.client()
         assert km.is_alive()
         is_done = km.ready.done()
         assert is_done
@@ -194,6 +202,7 @@ class TestKernelManager:
         assert km.is_alive()
         km.interrupt_kernel()
         assert isinstance(km, KernelManager)
+        kc.stop_channels()
         km.shutdown_kernel(now=True)
         assert km.context.closed
 
@@ -433,6 +442,7 @@ class TestParallel:
 
         km.shutdown_kernel()
         assert km.context.closed
+        kc.stop_channels()
 
 
 @pytest.mark.asyncio
